export class Main extends Indicator {

    constructor() {
        super();

        this.setProperty(Property.SeparateWindow, true);
        this.setProperty(Property.Minimum, 0);
        this.setProperty(Property.Maximum, 100);

        this.addLevel(75);
        this.addLevel(25);

        this.addInput("KPeriod", InputType.int, 5);
        this.addInput("DPeriod", InputType.int, 3);
        this.addInput("Slowing", InputType.int, 3);
        this.addInput("PriceType", InputType.PriceType, PriceType.Bid);

        this.buffers = {
            ind_buffer1 : this.addBuffer(),
        };

        this.arr = {
            MainBuffer : [],
            SignalBuffer : [],
            HighesBuffer : [],
            LowesBuffer : [],
        };

        this.vars = {
            draw_begin1: 0,
            draw_begin2: 0
        };
    }


  onInit() {
      const  {KPeriod, DPeriod, Slowing} = this.getInputs();
      let draw_begin1 = KPeriod + Slowing;

      this.buffers.ind_buffer1
            .setShape(Shape.Line)
            .setColor(Color.LightSeaGreen)
            .setLabel("Dynamo Sto(" + KPeriod + "," + DPeriod + "," + Slowing + ")")
            .setShortName("Dynamo Sto(" + KPeriod + "," + DPeriod + "," + Slowing + ")")
            .setDrawBegin(draw_begin1)
        ;

      this.vars.draw_begin2 = draw_begin1 + DPeriod;
      this.vars.draw_begin1 = draw_begin1;
  }

    async onUpdate() {
        const  {KPeriod, DPeriod, Slowing, PriceType} = this.getInputs(),
            {ind_buffer1} = this.buffers,
            {MainBuffer, SignalBuffer, HighesBuffer, LowesBuffer} = this.arr,
            {draw_begin1, draw_begin2} = this.vars,
            {
                High,
                Low,
                Close,
            } = await Bar.load([
                Bar.Mode.High,
                Bar.Mode.Low,
                Bar.Mode.Close,
            ], PriceType),
            barSize = Close.length;

        let i, k, countedBars = Bar.counted(), price, list;

        if (barSize <= draw_begin2) {
            return 0;
        }
        if (countedBars < 1) {
            for (i = 1; i <= draw_begin1; i++) {
                MainBuffer[barSize - i] = 0;
            }
            for (i = 1; i <= draw_begin2; i++) {
                SignalBuffer[barSize - i] = 0;
            }
        }
        i = barSize - KPeriod;
        if (countedBars > KPeriod) {
            i = barSize - countedBars - 1;
        }

        while (i >= 0) {
            let min = 1000000;
            k = i + KPeriod - 1;
            while (k >= i) {
                price = Low[k];
                if (min > price) min = price;
                k--;
            }
            LowesBuffer[i] = min;
            i--;
        }
        i = barSize - KPeriod;
        if (countedBars > KPeriod) {
            i = barSize - countedBars - 1;
        }
        while (i >= 0) {
            let max = -1000000;
            k = i + KPeriod - 1;
            while (k >= i) {
                price = High[k];
                if (max < price) max = price;
                k--;
            }
            HighesBuffer[i] = max;
            i--;
        }
        i = barSize - draw_begin1;
        if (countedBars > draw_begin1) {
            i = barSize - countedBars - 1;
        }
        while (i >= 0) {
            let sumlow = 0.0;
            let sumhigh = 0.0;
            for (k = (i + Slowing - 1); k >= i; k--) {
                sumlow += Close[k] - LowesBuffer[k];
                sumhigh += HighesBuffer[k] - LowesBuffer[k];
            }
            if (sumhigh === 0) {
                MainBuffer[i] = 100;
            }
            else {
                MainBuffer[i] = sumlow / sumhigh * 100;
            }
            i--;
        }
        if (countedBars > 0) countedBars--;
        let limit = barSize - countedBars;
        list = Indicators.MAOnArray(MainBuffer, barSize, DPeriod, 0, Averaging.Simple);
        for (i = 0; i < limit; i++) {
            SignalBuffer[i] = list[i];
        }
        for (i = 0; i < limit; i++) {
            ind_buffer1.set(i, (50 - SignalBuffer[i] + MainBuffer[i]));
        }

    }
}